
--
--  Gossipmonger
--    by Tuhljin
--

local Gossipmonger_Debug = 1

local THIS_VERSION = GetAddOnMetadata("Gossipmonger", "Version");

local CYAN = "|cff00ffff"
local YELLOW = "|cffffff00"
local YELLOW_LT = "|cffffff99"
local R, G, B = 0.494, 0.729, 1
local SCTColors = { r = R, g = G, b = B }

local minimumGossipHeight, maximumGossipHeight = 150, 414
local incGossipHeight = 135

--local R, G, B = 0.65, 0.75, 1
--local SCTColors = { r = 0.35, g = 0.55, b = 1 }
--local incGossipHeight = 90


Gossipmonger = {};

Gossipmonger_Settings = nil;
Gossipmonger.DefaultSettings = {
  Version = THIS_VERSION,
  skipIsDefault = true,
  informOnSkip_SCT = false,
  informOnSkip_log = false,
  alwaysInformWithName = false,
  showGossipMirrorOnSkip = true,
  delayShowingMirror = true,
  greetingModifierKey = 2,
};

-- LOCAL FUNCTIONS
-----------------------------------

local function chatprint(msg)
  DEFAULT_CHAT_FRAME:AddMessage(msg, R, G, B);
end

--[[
local function debugprint(msg)
  if (Gossipmonger_Debug >= 0) then
    chatprint("Gossip: "..msg)
  end
end
--]]

-- Table Helpers (tcopy and tcount) - by Mikk - from http://www.wowwiki.com/Table_Helpers
-- tcopy: recursively copy contents of one table to another
local function tcopy(to, from)   -- "to" must be a table (possibly empty)
 for k,v in pairs(from) do
   if(type(v)=="table") then
     to[k] = {}
     tcopy(to[k], v);
   else
     to[k] = v;
   end
 end
end

local function RegisterEvents_Show()
  Gossipmonger_MirrorFrame:RegisterEvent("BANKFRAME_OPENED");
  Gossipmonger_MirrorFrame:RegisterEvent("BATTLEFIELDS_SHOW");
  Gossipmonger_MirrorFrame:RegisterEvent("MERCHANT_SHOW");
  Gossipmonger_MirrorFrame:RegisterEvent("TAXIMAP_OPENED");
  Gossipmonger_MirrorFrame:RegisterEvent("TRAINER_SHOW");
end

local function RegisterEvents_Hide()
  Gossipmonger_MirrorFrame:RegisterEvent("BANKFRAME_CLOSED");
  Gossipmonger_MirrorFrame:RegisterEvent("BATTLEFIELDS_CLOSED");
  Gossipmonger_MirrorFrame:RegisterEvent("MERCHANT_CLOSED");
  Gossipmonger_MirrorFrame:RegisterEvent("TAXIMAP_CLOSED");
  Gossipmonger_MirrorFrame:RegisterEvent("TRAINER_CLOSED");
end

local function prepGossipOptionDisplay(titleButton, title, gossipType)
  titleButton:SetText(YELLOW..title.."   "..YELLOW_LT.."(selected)");
  GossipResize(titleButton);   -- Calls a Blizzard function that currently simply does this:  titleButton:SetHeight( titleButton:GetTextHeight() + 2);
  local titleButtonIcon = getglobal(titleButton:GetName() .. "GossipIcon")
  titleButtonIcon:SetTexture("Interface\\GossipFrame\\" .. gossipType .. "GossipIcon");
  titleButtonIcon:SetVertexColor(1, 1, 1, 1);
end

local function showGossipMirror(text, optionTitle, gossipType)
  Gossipmonger_MirrorFrame_GreetingText:SetText(text);
  prepGossipOptionDisplay(Gossipmonger_MirrorFrame_OptionDisplay, optionTitle, gossipType);

  local h = ( Gossipmonger_MirrorFrame_GreetingText:GetHeight() or 0 )
  if (h + incGossipHeight < minimumGossipHeight) then
    Gossipmonger_MirrorFrame:SetHeight(minimumGossipHeight)
    Gossipmonger_MirrorFrame_GreetingScrollFrameScrollBar:Hide()
  elseif (h <= Gossipmonger_MirrorFrame_GreetingScrollFrame:GetHeight() and h + incGossipHeight < maximumGossipHeight) then
    Gossipmonger_MirrorFrame:SetHeight(h + incGossipHeight)
    Gossipmonger_MirrorFrame_GreetingScrollFrameScrollBar:Hide()
  else
    Gossipmonger_MirrorFrame:SetHeight(maximumGossipHeight)
    Gossipmonger_MirrorFrame_GreetingScrollFrameScrollBar:Show()
  end

  local title = CYAN..( UnitName("npc") or "Gossipmonger" )
  Gossipmonger_MirrorFrame_NpcNameText:SetText(title);
  if ( UnitExists("npc") ) then
    SetPortraitTexture(Gossipmonger_MirrorFrame_Portrait, "npc");
  else
    Gossipmonger_MirrorFrame_Portrait:SetTexture("Interface\\QuestFrame\\UI-QuestLog-BookIcon");
  end

  Gossipmonger_MirrorFrame_GreetingScrollFrame:SetVerticalScroll(0);
  if (not Gossipmonger_Settings.delayShowingMirror) then
    ShowUIPanel(Gossipmonger_MirrorFrame)
  end
  RegisterEvents_Show()
end

local function skipModifierKeyHeld()
  if (Gossipmonger_Settings.greetingModifierKey == 0) then
    return IsAltKeyDown();
  elseif (Gossipmonger_Settings.greetingModifierKey == 1) then
    return IsControlKeyDown();
  elseif (Gossipmonger_Settings.greetingModifierKey == 2) then
    return IsShiftKeyDown();
  end
  -- If key is set to None (3), it always returns nil.
end

local function skipType(gossipType)
  if (gossipType == "gossip") then  return false;  end    -- Never skip "gossip" type.
  -- Possible future settings could determine which types of dialog options to skip. For now, skips all the standard
  -- ones that WoW normally does.
  return true;
end


-- GLOBAL FUNCTIONS
-----------------------------------

function Gossipmonger.ResetSettings(purge)
  if (purge) then   -- Purge unused variables. Generates garbage, so use only when necessary.
    Gossipmonger_Settings = {};
  end
  tcopy(Gossipmonger_Settings, Gossipmonger.DefaultSettings)
end

function Gossipmonger.OnLoad()
  tinsert(UISpecialFrames, "Gossipmonger_MirrorFrame");
  UIPanelWindows.Gossipmonger_MirrorFrame = {area = "right", pushable = 8, whileDead = 1};
  Gossipmonger_MirrorFrame:RegisterEvent("VARIABLES_LOADED");
end

function Gossipmonger.OnEvent()
  --debugprint(event)
  if (event == "VARIABLES_LOADED") then
    if (not Gossipmonger_Settings) then
      Gossipmonger_Settings = {};
      Gossipmonger.ResetSettings();
    else
      Gossipmonger_Settings.Version = THIS_VERSION;
    end
  elseif (event == "BANKFRAME_CLOSED" or event == "BATTLEFIELDS_CLOSED" or
          event == "MERCHANT_CLOSED" or event == "TAXIMAP_CLOSED" or event == "TRAINER_CLOSED") then
    if (Gossipmonger_MirrorFrame:IsShown()) then
      HideUIPanel(Gossipmonger_MirrorFrame)
    end
    Gossipmonger_MirrorFrame:UnregisterAllEvents();
  else
    if (not Gossipmonger_MirrorFrame:IsVisible()) then
      ShowUIPanel(Gossipmonger_MirrorFrame)
    end
    Gossipmonger_MirrorFrame:UnregisterAllEvents();
    RegisterEvents_Hide()
  end
end

function Gossipmonger.GoodbyeClick()
  HideUIPanel(Gossipmonger_MirrorFrame);
  if (BankFrame:IsVisible()) then
    HideUIPanel(BankFrame)
  elseif (BattlefieldFrame:IsVisible()) then
    HideUIPanel(BattlefieldFrame)
  elseif (MerchantFrame:IsVisible()) then
    HideUIPanel(MerchantFrame)
  elseif (TaxiFrame:IsVisible()) then
    HideUIPanel(TaxiFrame)
  elseif (ClassTrainerFrame:IsVisible()) then
    HideUIPanel(ClassTrainerFrame)
  end
end


-- HOOK
-----------------

local orig_GossipFrame_OnEvent = GossipFrame_OnEvent;
GossipFrame_OnEvent = function(...)
  if ( event == "GOSSIP_SHOW" and ( (GetNumGossipAvailableQuests() == 0) and (GetNumGossipActiveQuests() == 0) and (GetNumGossipOptions() == 1) ) ) then
-- See GossipFrame.lua in FrameXML.
    local optionTitle, gossipType = GetGossipOptions();

    -- Find instances where WoW normally skips gossip text.
    local keyheld = skipModifierKeyHeld()
    local skipthis = skipType(gossipType)
    if ( (Gossipmonger_Settings.skipIsDefault and skipthis and not keyheld) or (not Gossipmonger_Settings.skipIsDefault and skipthis and keyheld) ) then
      if (Gossipmonger_Settings.informOnSkip_SCT or Gossipmonger_Settings.informOnSkip_log) then
        local name = UnitName("npc")
        local title =  "The "..gossipType
        local text = " had something to say."
        local namelen = strlen(name)
        if ( Gossipmonger_Settings.alwaysInformWithName or (namelen <= 20 or namelen <= strlen (title)) ) then
          text = name..text
        else
          text = title..text
        end
        if (Gossipmonger_Settings.informOnSkip_SCT and SCT) then
          SCT:DisplayMessage( text, SCTColors )
        end
        if (Gossipmonger_Settings.informOnSkip_log) then
          chatprint(CYAN..text)
        end
      end
      SelectGossipOption(1);
      if (Gossipmonger_Settings.showGossipMirrorOnSkip) then
        showGossipMirror(GetGossipText(), optionTitle, gossipType)
      end
      return;
    end

    if ( not GossipFrame:IsShown() ) then
      ShowUIPanel(GossipFrame);
      if ( not GossipFrame:IsShown() ) then
        CloseGossip();
        return;
      end
    end
    GossipFrameUpdate();

  else
    --debugprint("normal ("..(event or "nil")..")")
    orig_GossipFrame_OnEvent(...)
  end
end
